@@ -8,7 +8,7 @@ BEGIN {
         $DB::options{ trace_returns }  //=  0;
         $DB::options{ _debug }      //=  0;
         $DB::options{ dbg_frames }  //=  0;
-        #@DB::options{ qw/ w s / } = ( 1, 1 );
+        @DB::options{ qw/ w s / } = ( 1, 1 );
         push @ISA, 'Devel::DebugHooks';
 }
 
@@ -39,7 +39,7 @@ my $last_input;
 sub interact {
 	my $self =  shift;
 
-	my $line =  <>; #$term->readline( 'DBG> ' );
+	my $line =  <STDIN>; #$term->readline( 'DBG> ' );
 	chomp $line;
 	if( $line ne '' ) {
 		$last_input =  $line;
